/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 1 $ */
/* $Date: 2/22/06 10:05a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux/HPDI32_Linux_Ver.1.13.0-GSC-DN/diphase2/lbtest/gpio.c $
 *
 * 1     2/22/06 10:05a Dnewbold
*/

#include <stdio.h>

#include "hpdi32_diphase2.h"
#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	_dir_hwhb_get_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Direction Get service for D31-D24.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_hwhb_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	errs	= 0;
	int	i;
	int	hwhb;
	int	hwlb;
	int	lwhb;
	int	lwlb;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_PO_31_24_ENABLE);
		errs	|= hpdi32_dp2_gpio_dir_get(fd, &hwhb, &hwlb, &lwhb, &lwlb);

		// Verify that D31-D24 changes as expected.
		errs	|= (hwhb == list[i].set) ? 0 : 1;

		// Verify that D23-D16 is unaffected.
		errs	|= hwlb ? 1 : 0;

		// Verify that D15-D8 is unaffected.
		errs	|= lwhb ? 1 : 0;

		// Verify that D7-D0 is unaffected.
		errs	|= lwlb ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_hwhb_set_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Direction Set service for D31-D24.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_hwhb_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 0,	0	},
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_gpio_dir_set(fd, list[i].set, -1, -1, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

		// Verify that D31-D24 changes as expected.
		enable	= (bcr & HPDI32_GSC_BCR_PO_31_24_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].get) ? 0 : 1;

		// Verify that D23-D16 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_23_16_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that D15-D8 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_15_08_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that D7-D0 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_07_00_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_hwhb_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's GPIO Direction feature for D31-D24.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_hwhb_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_PO_31_24_ENABLE);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_PO_31_24_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_PO_31_24_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_hwhb_tests
*
*	Purpose:
*
*		Perform tests on the library's D31-D24 GPIO Direction feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_hwhb_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("D31-D24 Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _dir_hwhb_get_tests(fd, verbose);
	errs	|= _dir_hwhb_set_tests(fd, verbose);
	errs	|= _dir_hwhb_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_hwlb_get_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Direction Get service for D23-D16.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_hwlb_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	errs	= 0;
	int	i;
	int	hwhb;
	int	hwlb;
	int	lwhb;
	int	lwlb;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_PO_23_16_ENABLE);
		errs	|= hpdi32_dp2_gpio_dir_get(fd, &hwhb, &hwlb, &lwhb, &lwlb);

		// Verify that D23-D16 changes as expected.
		errs	|= (hwlb == list[i].set) ? 0 : 1;

		// Verify that D31-D24 is unaffected.
		errs	|= hwhb ? 1 : 0;

		// Verify that D15-D8 is unaffected.
		errs	|= lwhb ? 1 : 0;

		// Verify that D7-D0 is unaffected.
		errs	|= lwlb ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_hwlb_set_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Direction Set service for D23-D16.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_hwlb_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 0,	0	},
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_gpio_dir_set(fd, -1, list[i].set, -1, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

		// Verify that D23-D16 changes as expected.
		enable	= (bcr & HPDI32_GSC_BCR_PO_23_16_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].get) ? 0 : 1;

		// Verify that D31-D24 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_31_24_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that D15-D8 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_15_08_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that D7-D0 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_07_00_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_hwlb_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's GPIO Direction feature for D23-D16.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_hwlb_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_PO_23_16_ENABLE);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_PO_23_16_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_PO_23_16_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_hwlb_tests
*
*	Purpose:
*
*		Perform tests on the library's D23-D16 GPIO Direction feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_hwlb_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("D23-D16 Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _dir_hwlb_get_tests(fd, verbose);
	errs	|= _dir_hwlb_set_tests(fd, verbose);
	errs	|= _dir_hwlb_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_lwhb_get_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Direction Get service for D15-D8.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_lwhb_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	errs	= 0;
	int	i;
	int	hwhb;
	int	hwlb;
	int	lwhb;
	int	lwlb;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_PO_15_08_ENABLE);
		errs	|= hpdi32_dp2_gpio_dir_get(fd, &hwhb, &hwlb, &lwhb, &lwlb);

		// Verify that D15-D8 changes as expected.
		errs	|= (lwhb == list[i].set) ? 0 : 1;

		// Verify that D31-D24 is unaffected.
		errs	|= hwhb ? 1 : 0;

		// Verify that D23-D16 is unaffected.
		errs	|= hwlb ? 1 : 0;

		// Verify that D7-D0 is unaffected.
		errs	|= lwlb ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_lwhb_set_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Direction Set service for D15-D8.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_lwhb_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 0,	0	},
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_gpio_dir_set(fd, -1, -1, list[i].set, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

		// Verify that D15-D8 changes as expected.
		enable	= (bcr & HPDI32_GSC_BCR_PO_15_08_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].get) ? 0 : 1;

		// Verify that D31-D24 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_31_24_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that D23-D16 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_23_16_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that D7-D0 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_07_00_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_lwhb_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's GPIO Direction feature for D15-D8.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_lwhb_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_PO_15_08_ENABLE);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_PO_15_08_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_PO_15_08_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_lwhb_tests
*
*	Purpose:
*
*		Perform tests on the library's D15-D8 GPIO Direction feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_lwhb_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("D15-D8 Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _dir_lwhb_get_tests(fd, verbose);
	errs	|= _dir_lwhb_set_tests(fd, verbose);
	errs	|= _dir_lwhb_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_lwlb_get_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Direction Get service for D15-D8.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_lwlb_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	errs	= 0;
	int	i;
	int	hwhb;
	int	hwlb;
	int	lwhb;
	int	lwlb;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_PO_07_00_ENABLE);
		errs	|= hpdi32_dp2_gpio_dir_get(fd, &hwhb, &hwlb, &lwhb, &lwlb);

		// Verify that D7-D0 changes as expected.
		errs	|= (lwlb == list[i].set) ? 0 : 1;

		// Verify that D31-D24 is unaffected.
		errs	|= hwhb ? 1 : 0;

		// Verify that D23-D16 is unaffected.
		errs	|= hwlb ? 1 : 0;

		// Verify that D15-D8 is unaffected.
		errs	|= lwhb ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_lwlb_set_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Direction Set service for D7-D0.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_lwlb_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 0,	0	},
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_gpio_dir_set(fd, -1, -1, -1, list[i].set);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

		// Verify that D7-D0 changes as expected.
		enable	= (bcr & HPDI32_GSC_BCR_PO_07_00_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].get) ? 0 : 1;

		// Verify that D31-D24 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_31_24_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that D23-D16 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_23_16_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that D15-D8 is unchanged.
		enable	= (bcr & HPDI32_GSC_BCR_PO_15_08_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_lwlb_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's GPIO Direction feature for D7-D0.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_lwlb_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_PO_07_00_ENABLE);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_PO_07_00_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_PO_07_00_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_dir_lwlb_tests
*
*	Purpose:
*
*		Perform tests on the library's D7-D0 GPIO Direction feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dir_lwlb_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("D7-D0 Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _dir_lwlb_get_tests(fd, verbose);
	errs	|= _dir_lwlb_set_tests(fd, verbose);
	errs	|= _dir_lwlb_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_direction_tests
*
*	Purpose:
*
*		Perform tests on the library's GPIO Direction feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _direction_tests(int fd)
{
	int	errs	= 0;

	label("Direction");
	printf("\n");

	label_level_inc();

	errs	|= _dir_hwhb_tests(fd);
	errs	|= _dir_hwlb_tests(fd);
	errs	|= _dir_lwhb_tests(fd);
	errs	|= _dir_lwlb_tests(fd);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	_output_write_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Output Write feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _output_write_tests(int fd, int verbose)
{
	__u32	bit;
	int		errs	= 0;
	int		i;
	__u32	txodr;
	__u32	value;

	if (verbose)
		label("Write Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	// Walking 1.

	for (i = 0; i <= 31; i++)
	{
		bit		= 1L << i;
		value	= bit;
		errs	|= hpdi32_dp2_gpio_write(fd, value);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXODR, &txodr);
		errs	|= (txodr == value) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	// Walking 0.

	for (i = 0; i <= 31; i++)
	{
		bit		= 1L << i;
		value	= ~bit;
		errs	|= hpdi32_dp2_gpio_write(fd, value);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXODR, &txodr);
		errs	|= (txodr == value) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_output_mod_tests
*
*	Purpose:
*
*		Perform tests of the library's GPIO Output Read-Modify-Write feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _output_mod_tests(int fd, int verbose)
{
	__u32	bit;
	int		errs	= 0;
	int		i;
	__u32	txodr;
	__u32	value;

	if (verbose)
		label("Modify Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	// Walking 1.

	for (i = 0; i <= 31; i++)
	{
		errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TXODR, 0);
		bit		= 1L << i;
		value	= bit;
		errs	|= hpdi32_dp2_gpio_mod(fd, ~0L, bit);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXODR, &txodr);
		errs	|= (txodr == value) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	// Walking 0.

	for (i = 0; i <= 31; i++)
	{
		errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TXODR, ~0L);
		bit		= 1L << i;
		value	= ~bit;
		errs	|= hpdi32_dp2_gpio_mod(fd, 0, bit);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXODR, &txodr);
		errs	|= (txodr == value) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_output_reset_tests
*
*	Purpose:
*
*		Perform Reset tests of the library's GPIO Output feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _output_reset_tests(int fd, int verbose)
{
	__u32	bit;
	int		errs	= 0;
	int		i;
	__u32	txodr;

	if (verbose)
		label("Reset");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; i <= 31; i++)
	{
		// Apply a setting.
		bit		= 1L << i;

		// Verify that the setting is cleared by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TXODR, bit);
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXODR, &txodr);
		errs	|= txodr ? 1 : 0;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TXODR, bit);
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXODR, &txodr);
		errs	|= txodr ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_output_tests
*
*	Purpose:
*
*		Perform tests on the library's GPIO Output feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _output_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Output Latches");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _output_write_tests(fd, verbose);
	errs	|= _output_mod_tests(fd, verbose);
	errs	|= _output_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	gpio_tests
*
*	Purpose:
*
*		Perform tests on the GPIO feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int gpio_tests(int fd)
{
	int		errs	= 0;

	label("GPIO Tests");
	printf("\n");
	label_level_inc();

	errs	|= _direction_tests(fd);
	errs	|= _output_tests(fd);

	label_level_dec();
	return(errs);
}



