/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 2 $ */
/* $Date: 9/28/06 5:51p $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.17.0-GSC-DN/diphase2/lbtest/loopback.c $
 *
 * 2     9/28/06 5:51p Dnewbold
 * Added code to support the Data Size and PIO Threshold of the 1.18
 * driver.
 *
 * 1     2/22/06 10:05a Dnewbold
*/

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "hpdi32_diphase2.h"
#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	_lb_io_test
*
*	Purpose:
*
*		Perform a data loopback test using the given I/O mode.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		mode	The I/O mode to utilize.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _lb_io_test(int fd, int mode)
{
	#define	LOOPS		100
	#define	SAMPLES		512	// No bigger than the smallest FIFO.

	__u32				bsr;
	int					errs	= 0;
	int					i;
	hpdi32_io_config_t	io;
	int					loop;
	int					report		= 1;
	__u32				rx_buf[SAMPLES];
	int					rx_qty;
	long				rx_tot	= 0;
	__u32				tx_buf[SAMPLES];
	int					tx_qty;
	long				tx_tot	= 0;

	for (i = 0; i < SAMPLES; i++)
		tx_buf[i]	= 0xAAAA0000L + i;

	errs	|= hpdi32_dp2_board_reset(fd);

	memset(&io, 0, sizeof(io));
	io.read.mode			= mode;
	io.read.timeout			= 5;
	io.read.samples			= SAMPLES;
	io.read.sample_bits		= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.read.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	io.write.mode			= mode;
	io.write.timeout		= 5;
	io.write.samples		= SAMPLES;
	io.write.sample_bits	= HPDI32_IO_SAMPLE_BITS_NO_CHANGE;
	io.write.pio_threshold	= HPDI32_IO_PIO_THRESHOLD_NO_CHANGE;
	errs	|= hpdi32_io_config(fd, &io, 0);

	errs	|= hpdi32_dp2_predrive_set(fd, 0, 0, 0);
	errs	|= hpdi32_dp2_tx_config_set(fd, 1, 0, 0, 0, 0);
	errs	|= hpdi32_dp2_board_loop_back_set(fd, 1);
	errs	|= hpdi32_dp2_rx_config_set(fd, 1, 0, 0, 1);

	for (loop = 0; (errs == 0) && (loop < LOOPS); loop++)
	{
		// Reset things in preparation for data transfers.
		errs	|= HPDI32_DP2__TX_DISABLE(fd);
		errs	|= HPDI32_DP2__TX_STOP(fd);
		errs	|= HPDI32_DP2__RX_DISABLE(fd);
		// All three must be set before write() for DMDMA loop back.
		errs	|= HPDI32_DP2__TX_ENABLE(fd);
		errs	|= HPDI32_DP2__TX_START(fd);
		errs	|= HPDI32_DP2__RX_ENABLE(fd);

		tx_qty	= write(fd, tx_buf, sizeof(tx_buf));

		if (tx_qty != sizeof(tx_buf))
		{
			report	= 0;
			printf(	"FAIL <---  (write returned %d, errno %d)\n",
					tx_qty,
					errno);
			break;
		}

		memset(rx_buf, 0, sizeof(rx_buf));
		tx_tot	+= tx_qty;

		// Wait for the Tx FIFO to empty. This is necessary for non-Demand
		// Mode DMA so that all of the data is present when we do the read.
		// This is also necessary with Demand Mode DMA since the board won't
		// do Rx DMDMA with the transmitter active.

		for (i = 0; i < 1000; i++)
		{
			errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);

			if ((bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) == 0)
				break;

			usleep(1000);
		}

		if (i >= 1000)
		{
			report	= 0;
			printf("FAIL <---  (data Tx failed to complete)\n");
			break;
		}

		usleep(1000);	// Wait for transmitter to empty.
		rx_qty		= read(fd, rx_buf, sizeof(rx_buf));

		if (rx_qty != sizeof(rx_buf))
		{
			report	= 0;
			printf(	"FAIL <---  (read returned %d, errno %d)\n",
					rx_qty,
					errno);
			break;
		}

		rx_tot	+= rx_qty;

		if (memcmp(tx_buf, rx_buf, sizeof(tx_buf)))
		{
			report	= 0;
			printf("FAIL <---  (data mismatch)\n");
			break;
		}
	}

	if (report == 0)
		errs	|= 1;
	else if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS  (Tx %ld bytes, Rx %ld bytes)\n", tx_tot, rx_tot);

	hpdi32_dp2_board_reset(fd);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_lb_dma_test
*
*	Purpose:
*
*		Perform a non-Demand Mode DMA data loopback test.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _lb_dma_test(int fd)
{
	int	errs;

	label("DMA");
	errs	= _lb_io_test(fd, HPDI32_IO_MODE_DMA);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_lb_dmdma_test
*
*	Purpose:
*
*		Perform a Demand Mode DMA data loopback test.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _lb_dmdma_test(int fd)
{
	int	errs;

	label("DMDMA");
	errs	= _lb_io_test(fd, HPDI32_IO_MODE_DMDMA);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_lb_pio_test
*
*	Purpose:
*
*		Perform a PIO data loopback test.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _lb_pio_test(int fd)
{
	int	errs;

	label("PIO");
	errs	= _lb_io_test(fd, HPDI32_IO_MODE_PIO);
	return(errs);
}



/*****************************************************************************
*
*	Function:	loopback_test
*
*	Purpose:
*
*		Perform a data loopback test.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int loopback_test(int fd)
{
	int		errs	= 0;

	label("Loop Back Test");
	printf("\n");
	label_level_inc();

	errs	|= _lb_pio_test(fd);
	errs	|= _lb_dma_test(fd);
	errs	|= _lb_dmdma_test(fd);

	label_level_dec();
	return(errs);
}



