/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 1 $ */
/* $Date: 2/22/06 10:05a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux/HPDI32_Linux_Ver.1.13.0-GSC-DN/diphase2/lbtest/main.c $
 *
 * 1     2/22/06 10:05a Dnewbold
*/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "hpdi32_diphase2.h"
#include "main.h"
#include "hpdi32_dp2_dsl.h"
#include "HPDI32DocSrcLib.h"
#include "util_count.h"
#include "util_id.h"
#include "util_lib_id.h"
#include "util_label.h"
#include "util_select.h"



/* variables	*************************************************************/

static	int		_continuous		= 0;
static	int		_def_board		= -1;
static	int		_ignore_errors	= 0;
static	long	_max_tests		= 0;
static	long	_minute_limit	= 0;
static	int		_quit			= 0;



/*****************************************************************************
*
*	Function:	_parse_args
*
*	Purpose:
*
*		Parse the command line arguments.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _parse_args(int argc, char** argv)
{
	int	i;

	printf("USAGE: lbtest <-c> <-C> <-n#> <#>\n");
	printf("   -c   Repeat test until an error occurs.\n");
	printf("   -C   Repeat test even if errors occur.\n");
	printf("   -m#  Run for at most # minutes (a decimal number).\n");
	printf("   -n#  Repeat test at most # times (a decimal number).\n");
	printf("   #    The default board index to test.\n");

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-c") == 0)
		{
			_continuous		= 1;
			_ignore_errors	= 0;
			continue;
		}

		if (strcmp(argv[i], "-C") == 0)
		{
			_continuous		= 1;
			_ignore_errors	= 1;
			continue;
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 'm') &&
			(isdigit(argv[i][2])))
		{
			sscanf(argv[i] + 2, "%ld", &_minute_limit);

			if (_minute_limit < 0)
			{
				printf("ERROR: invalid count: %s\n", argv[i]);
				break;
			}

			continue;
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 'n') &&
			(isdigit(argv[i][2])))
		{
			sscanf(argv[i] + 2, "%ld", &_max_tests);

			if (_max_tests < 0)
			{
				printf("ERROR: invalid count: %s\n", argv[i]);
				break;
			}

			continue;
		}

		if (isdigit(argv[i][0]) == 0)
		{
			_quit	= 1;
			printf("ERROR: invalid arguments: %s\n", argv[i]);
			break;
		}

		sscanf(argv[i], "%d", &_def_board);
	}
}



/*****************************************************************************
*
*	Function:	_perform_tests
*
*	Purpose:
*
*		Perform the appropriate testing.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem, see errno.
*
*****************************************************************************/

static int _perform_tests(int fd)
{
	const char*	psz;
	int			status	= 0;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= asctime(stm);
	label("Performing Test");
	printf("%s", psz);
	id_host();
	status	|= id_driver(fd);
	id_dp2_library();
	status	|= id_board(fd);
	status	|= loopback_test(fd);

	label("RESULTS");
	printf("%s\n", status ? "FAIL" : "PASS");
	status	= status ? -1 : 0;
	return(status);
}



/*****************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Program entry point.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	All went well.
*		EXIT_FAILURE	There was a problem.
*
*****************************************************************************/

int main(int argc, char **argv)
{
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	int		fd;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;
	int		qty;
	int		status		= 0;

	label_init(26);

	for (;;)
	{
		test	= time(NULL);
		printf("HPDI32 Test Utility (Version %s)\n", VERSION);
		_parse_args(argc, argv);

		if (_quit)
			break;

		t_limit	= exec + (_minute_limit * 60);
		qty		= count_boards();

		if (qty <= 0)
			break;

		errs	= select_1_board(qty, &_def_board);

		if (errs)
			break;

		label("Testing Board Index");
		printf("%d\n", _def_board);
		fd	= hpdi32_open(_def_board, 0);

		if (fd == -1)
		{
			status	= -1;
		}
		else
		{
			status	= _perform_tests(fd);
			hpdi32_close(fd, 0);
		}

		now	= time(NULL);
		tests++;

		if (status)
		{
			failures++;
			printf("\nRESULTS: FAIL <---");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= now - test;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= now - exec;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (!_continuous)
			break;

		if ((status) && (!_ignore_errors))
			break;

		if ((_max_tests) && (tests >= _max_tests))
			break;

		if ((_minute_limit) && (now >= t_limit))
			break;
	}

	ret	= status ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}



