/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
#include <errno.h>
#include <stdio.h>
#include <sys/ioctl.h>

#include "hpdi32_diphase2.h"



/******************************************************************************
*
*	Function:	hpdi32_dp2_predrive_get
*
*	Purpose:
*
*		Retrieve the current predrive configuration.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		enable	This is set to one if predrive is enabled, and zero otherwise.
*
*		high	This is set to one if predrive is gifured a HIGH and zero ig
*				configured as low.
*
*		count	This is set to the current count configuration.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_predrive_get(int fd, int* enable, int* high, int* count)
{
	__u32	bcr;
	int		err;
	__u32	txpdlr;

	err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);

	if (err == 0)
		err	= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXPDLR, &txpdlr);

	if (enable == NULL)
		;
	else if (bcr & HPDI32_GSC_BCR_TX_PD_ENABLE)
		enable[0]	= 1;
	else
		enable[0]	= 0;

	if (high == NULL)
		;
	else if (bcr & HPDI32_GSC_BCR_TX_PD_LEVEL)
		high[0]	= 1;
	else
		high[0]	= 0;

	if (count)
		count[0]	= (int) (0xFFFF & txpdlr);

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_predrive_set
*
*	Purpose:
*
*		Update the current predrive configuration.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		enable	Enable predrive? <0 = ignore, 0 = disable, >0 = enable
*				If the count argument is zero, this argument will be ignored,
*				and predrive will be disabled.
*
*		high	Ouput high level? <0 = ignore, 0 = low, >0 = high
*
*		count	The count to apply. <0 = ignore, 0 = disable predrive,
*				>0 set to this value
*				A value of zero will NOT be applied. Instead, it will result
*				in predrive being disabled. A value over 0xFFFF will produce
*				an error.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_predrive_set(int fd, int enable, int high, int count)
{
	#define	BCR_MOD(v,m)	hpdi32_dp2_reg_mod(fd, HPDI32_GSC_BCR, (v), (m))

	int	err	= 0;

	if (count > 0xFFFF)
		err	= EINVAL;

	if (count == 0)
		enable	= 0;

	if ((err) || (enable < 0))
		;
	else if (enable)
		err	= BCR_MOD(~0L, HPDI32_GSC_BCR_TX_PD_ENABLE);
	else
		err	= BCR_MOD(0, HPDI32_GSC_BCR_TX_PD_ENABLE);

	if ((err) || (high < 0))
		;
	else if (high)
		err	= BCR_MOD(~0L, HPDI32_GSC_BCR_TX_PD_LEVEL);
	else
		err	= BCR_MOD(0, HPDI32_GSC_BCR_TX_PD_LEVEL);

	if ((err) || (count <= 0))
		;
	else
		err	= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TXPDLR, count);

	return(err);
}



