/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
#include <errno.h>
#include <stdio.h>
#include <sys/ioctl.h>

#include "hpdi32_diphase2.h"



/******************************************************************************
*
*	Function:	hpdi32_dp2_reg_mod
*
*	Purpose:
*
*		Perform a read-modify-write on the given registers.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		reg		The register to access.
*
*		value	The value to apply to the register.
*
*		mask	The set of bits to modify. If a bit is set, then that bit
*				will be updated with the correspong "value" bit. If a bit is
*				zero, the the corresponding bit will not be modified.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_reg_mod(int fd, __u32 reg, __u32 value, __u32 mask)
{
	int				err;
	hpdi32_reg_t	parm;

	parm.reg	= reg;
	parm.value	= value;
	parm.mask	= mask;
	err			= ioctl(fd, HPDI32_IOCTL_REG_MOD, &parm);
	err			= err ? errno : 0;
	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_reg_read
*
*	Purpose:
*
*		Read the value of the specified register.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		reg		The register to access.
*
*		value	The value read from the register is stored here.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_reg_read(int fd, __u32 reg, __u32* value)
{
	int				err;
	hpdi32_reg_t	parm;

	parm.reg	= reg;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0;
	err			= ioctl(fd, HPDI32_IOCTL_REG_READ, &parm);
	err			= err ? errno : 0;

	if (value)
		value[0]	= parm.value;

	return(err);
}



/******************************************************************************
*
*	Function:	hpdi32_dp2_reg_write
*
*	Purpose:
*
*		Write the given value to the specified register.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*		reg		The register to access.
*
*		value	The value to apply to the register.
*
*	Returned:
*
*		0		All went well.
*		!0		The "errno" value correspondign to any error that occurred.
*
******************************************************************************/

int hpdi32_dp2_reg_write(int fd, __u32 reg, __u32 value)
{
	int				err;
	hpdi32_reg_t	parm;

	parm.reg	= reg;
	parm.value	= value;
	parm.mask	= 0;
	err			= ioctl(fd, HPDI32_IOCTL_REG_WRITE, &parm);
	err			= err ? errno : 0;
	return(err);
}



