/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 1 $ */
/* $Date: 2/22/06 10:08a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux/HPDI32_Linux_Ver.1.13.0-GSC-DN/diphase2/sbtest/loopback.c $
 *
 * 1     2/22/06 10:08a Dnewbold
*/

#include <stdio.h>

#include "hpdi32_diphase2.h"
#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	_enable_get_tests
*
*	Purpose:
*
*		Perform tests of the library's Loop Back Get service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _enable_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ 0,	0	},
		{ 1,	1	},
		{ 0,	0	},
		{ 1,	1	},
		{ 0,	0	},
		{ -1,	-1	}
	};

	int	enable;
	int	errs	= 0;
	int	i;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_LOOP_BACK);
		errs	|= hpdi32_dp2_board_loop_back_get(fd, &enable);
		errs	|= (enable == list[i].get) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_enable_set_tests
*
*	Purpose:
*
*		Perform tests of the library's Loop Back Set service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _enable_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 0,	0	},
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		errs	|= hpdi32_dp2_board_loop_back_set(fd, list[i].set);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_LOOP_BACK) ? 1 : 0;
		errs	|= (enable == list[i].get) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_enable_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's Loop Back feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _enable_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_LOOP_BACK);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_LOOP_BACK) ? 1 : 0;
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_LOOP_BACK) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_enable_tests
*
*	Purpose:
*
*		Perform tests on the library's loop back enable feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _enable_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Enable Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _enable_get_tests(fd, verbose);
	errs	|= _enable_set_tests(fd, verbose);
	errs	|= _enable_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	loopback_tests
*
*	Purpose:
*
*		Perform tests on the library's loop back feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int loopback_tests(int fd)
{
	int		errs	= 0;

	label("Loop Back Tests");
	printf("\n");
	label_level_inc();

	errs	|= _enable_tests(fd);

	label_level_dec();
	return(errs);
}



