/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 1 $ */
/* $Date: 2/22/06 10:08a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux/HPDI32_Linux_Ver.1.13.0-GSC-DN/diphase2/sbtest/predrive.c $
 *
 * 1     2/22/06 10:08a Dnewbold
*/

#include <stdio.h>

#include "hpdi32_diphase2.h"
#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	_count_get_tests
*
*	Purpose:
*
*		Perform tests of the library's PreDrive Length Get service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _count_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 10	},
		{ 100	},
		{ 1000	},
		{ 104	},
		{ -1	}
	};

	int	count;
	int	enable;
	int	errs	= 0;
	int	high;
	int	i;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TXPDLR,
										list[i].set,
										0xFFFF);
		errs	|= hpdi32_dp2_predrive_get(fd, &enable, &high, &count);

		// Verify that the Count setting changes as expected.
		errs	|= (count == list[i].set) ? 0 : 1;

		// Verify that the Enable setting is unaffected.
		errs	|= enable ? 1 : 0;

		// Verify that the High setting is unaffected.
		errs	|= high ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_count_set_tests
*
*	Purpose:
*
*		Perform tests of the library's PreDrive Length Set service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _count_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set		get
		{ -1,		104		},
		{ 1,		1		},
		{ -1,		1		},
		{ 10,		10		},
		{ -1,		10		},
		{ 100,		100		},
		{ 0xFFFF,	0xFFFF	},
		{ -1,		0xFFFF	},
		{ -1,		-1		}
	};

	__u32	bcr;
	int		count;
	int		enable;
	int		errs	= 0;
	int		high;
	int		i;
	__u32	txpdlr;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_predrive_set(fd, -1, -1, list[i].set);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXPDLR, &txpdlr);

		// Verify that the Count setting changes as expected.
		count	= txpdlr & 0xFFFF;
		errs	|= (count == list[i].get) ? 0 : 1;

		// Verify that the Enable setting is unaffected.
		enable	= (bcr & HPDI32_GSC_BCR_TX_PD_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that the High setting is unaffected.
		high	= (bcr & HPDI32_GSC_BCR_TX_PD_LEVEL) ? 1 : 0;
		errs	|= high ? 1 : 0;
	}

	// Verify that applying a count of zero disables predrive and
	// that the count remains unchanged.
	errs	|= hpdi32_dp2_predrive_set(fd, 1, -1, 1000);
	errs	|= hpdi32_dp2_predrive_set(fd, 1, -1, 0);
	errs	|= hpdi32_dp2_predrive_get(fd, &enable, &high, &count);
	errs	|= enable ? 1 : 0;
	errs	|= (count == 1000) ? 0 : 1;

	// Verify that a count over 0xFFFF is invalid.
	errs	|= hpdi32_dp2_predrive_set(fd, 0, 0, 0x10000) ? 0 : 1;

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_count_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's PreDrive Length feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _count_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 10	},
		{ 100	},
		{ 1000	},
		{ -1	}
	};

	int		count;
	int		errs	= 0;
	int		i;
	__u32	txpdlr;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply the setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TXPDLR,
										list[i].set,
										0xFFFF);

		// Verify that the setting is reset by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXPDLR, &txpdlr);
		count	= txpdlr & 0xFFFF;
		errs	|= (count == 104) ? 0 : 1;

		// Apply the setting again.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TXPDLR,
										list[i].set,
										0xFFFF);

		// Verify that the setting is reset by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXPDLR, &txpdlr);
		count	= txpdlr & 0xFFFF;
		errs	|= (count == 104) ? 0 : 1;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_count_tests
*
*	Purpose:
*
*		Perform tests on the library's PreDrive Length feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _count_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Length Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _count_get_tests(fd, verbose);
	errs	|= _count_set_tests(fd, verbose);
	errs	|= _count_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_enable_get_tests
*
*	Purpose:
*
*		Perform tests of the library's PreDrive Enable Get service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _enable_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	count;
	int	enable;
	int	errs	= 0;
	int	high;
	int	i;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_PD_ENABLE);
		errs	|= hpdi32_dp2_predrive_get(fd, &enable, &high, &count);

		// Verify that the Enable setting changes as expected.
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that the High setting is unaffected.
		errs	|= high ? 1 : 0;

		// Verify that the Count setting is unaffected.
		errs	|= (count == 104) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_enable_set_tests
*
*	Purpose:
*
*		Perform tests of the library's PreDrive Enable Set service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _enable_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 0,	0	},
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		count;
	int		enable;
	int		errs	= 0;
	int		high;
	int		i;
	__u32	txpdlr;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_predrive_set(fd, list[i].set, -1, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXPDLR, &txpdlr);

		// Verify that the Enable setting changes as expected.
		enable	= (bcr & HPDI32_GSC_BCR_TX_PD_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].get) ? 0 : 1;

		// Verify that the High setting is unaffected.
		high	= (bcr & HPDI32_GSC_BCR_TX_PD_LEVEL) ? 1 : 0;
		errs	|= high ? 1 : 0;

		// Verify that the Count setting is unaffected.
		count	= txpdlr & 0xFFFF;
		errs	|= (count == 104) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_enable_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's PreDrive Enable feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _enable_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_PD_ENABLE);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_TX_PD_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_TX_PD_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_enable_tests
*
*	Purpose:
*
*		Perform tests on the library's PreDrive Enable feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _enable_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Enable Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _enable_get_tests(fd, verbose);
	errs	|= _enable_set_tests(fd, verbose);
	errs	|= _enable_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_level_get_tests
*
*	Purpose:
*
*		Perform tests of the library's PreDrive Level Get service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _level_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	count;
	int	enable;
	int	errs	= 0;
	int	high;
	int	i;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_PD_LEVEL);
		errs	|= hpdi32_dp2_predrive_get(fd, &enable, &high, &count);

		// Verify that the High setting changes as expected.
		errs	|= (high == list[i].set) ? 0 : 1;

		// Verify that the Enable setting is unaffected.
		errs	|= enable ? 1 : 0;

		// Verify that the Count setting is unaffected.
		errs	|= (count == 104) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_level_set_tests
*
*	Purpose:
*
*		Perform tests of the library's PreDrive Level Set service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _level_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 0,	0	},
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		count;
	int		enable;
	int		errs	= 0;
	int		high;
	int		i;
	__u32	txpdlr;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_predrive_set(fd, -1, list[i].set, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXPDLR, &txpdlr);

		// Verify that the High setting changes as expected.
		high	= (bcr & HPDI32_GSC_BCR_TX_PD_LEVEL) ? 1 : 0;
		errs	|= (high == list[i].get) ? 0 : 1;

		// Verify that the Enable setting is unaffected.
		enable	= (bcr & HPDI32_GSC_BCR_TX_PD_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that the Count setting is unaffected.
		count	= txpdlr & 0xFFFF;
		errs	|= (count == 104) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_level_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's PreDrive Level feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _level_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		errs	= 0;
	int		high;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_PD_LEVEL);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		high	= (bcr & HPDI32_GSC_BCR_TX_PD_LEVEL) ? 1 : 0;
		errs	|= (high == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		high	= (bcr & HPDI32_GSC_BCR_TX_PD_LEVEL) ? 1 : 0;
		errs	|= high ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_level_tests
*
*	Purpose:
*
*		Perform tests on the library's PreDrive Level feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _level_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Level Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _level_get_tests(fd, verbose);
	errs	|= _level_set_tests(fd, verbose);
	errs	|= _level_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	predrive_tests
*
*	Purpose:
*
*		Perform tests on the PreDrive feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int predrive_tests(int fd)
{
	int		errs	= 0;

	label("PreDrive Tests");
	printf("\n");
	label_level_inc();

	errs	|= _enable_tests(fd);
	errs	|= _level_tests(fd);
	errs	|= _count_tests(fd);

	label_level_dec();
	return(errs);
}



