/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 1 $ */
/* $Date: 2/22/06 10:08a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32_Linux/HPDI32_Linux_Ver.1.13.0-GSC-DN/diphase2/sbtest/reg.c $
 *
 * 1     2/22/06 10:08a Dnewbold
*/

#include <stdio.h>

#include "hpdi32_diphase2.h"
#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/* data types	*************************************************************/

typedef struct
{
	const char*	name;
	__u32		reg;
} reg_entry_t;



/* variables	*************************************************************/

#define	PCI_REG(r)	#r, HPDI32_PCI_##r

static const reg_entry_t	_pci_regs[]	=
{
	{ PCI_REG(IDR)		},
	{ PCI_REG(CR)		},
	{ PCI_REG(SR)		},
	{ PCI_REG(REV)		},
	{ PCI_REG(CCR)		},
	{ PCI_REG(CLSR)		},
	{ PCI_REG(LTR)		},
	{ PCI_REG(HTR)		},
	{ PCI_REG(BISTR)	},
	{ PCI_REG(BAR0)		},
	{ PCI_REG(BAR1)		},
	{ PCI_REG(BAR2)		},
	{ PCI_REG(BAR3)		},
	{ PCI_REG(BAR4)		},
	{ PCI_REG(BAR5)		},
	{ PCI_REG(CIS)		},
	{ PCI_REG(SVID)		},
	{ PCI_REG(SID)		},
	{ PCI_REG(ERBAR)	},
	{ PCI_REG(ILR)		},
	{ PCI_REG(IPR)		},
	{ PCI_REG(MGR)		},
	{ PCI_REG(MLR)		},
	{ NULL, 0			}
};

#define	PLX_REG(r)	#r, HPDI32_PLX_##r

static const reg_entry_t	_plx_regs[]	=
{
	{ PLX_REG(LAS0RR)	},
	{ PLX_REG(LAS0BA)	},
	{ PLX_REG(MARBR)	},
	{ PLX_REG(BIGEND)	},
	{ PLX_REG(EROMRR)	},
	{ PLX_REG(EROMBA)	},
	{ PLX_REG(LBRD0)	},
	{ PLX_REG(DMRR)		},
	{ PLX_REG(DMLBAM)	},
	{ PLX_REG(DMLBAI)	},
	{ PLX_REG(DMPBAM)	},
	{ PLX_REG(DMCFGA)	},
	{ PLX_REG(LAS1RR)	},
	{ PLX_REG(LAS1BA)	},
	{ PLX_REG(LBRD1)	},

	{ PLX_REG(MBOX0)	},
	{ PLX_REG(MBOX1)	},
	{ PLX_REG(MBOX2)	},
	{ PLX_REG(MBOX3)	},
	{ PLX_REG(MBOX4)	},
	{ PLX_REG(MBOX5)	},
	{ PLX_REG(MBOX6)	},
	{ PLX_REG(MBOX7)	},
	{ PLX_REG(P2LDBELL)	},
	{ PLX_REG(L2PDBELL)	},
	{ PLX_REG(INTCSR)	},
	{ PLX_REG(CNTRL)	},
	{ PLX_REG(PCIHIDR)	},
	{ PLX_REG(PCIHREV)	},

	{ PLX_REG(DMAMODE0)	},
	{ PLX_REG(DMAPADR0)	},
	{ PLX_REG(DMALADR0)	},
	{ PLX_REG(DMASIZ0)	},
	{ PLX_REG(DMADPR0)	},
	{ PLX_REG(DMAMODE1)	},
	{ PLX_REG(DMAPADR1)	},
	{ PLX_REG(DMALADR1)	},
	{ PLX_REG(DMASIZ1)	},
	{ PLX_REG(DMADPR1)	},
	{ PLX_REG(DMACSR0)	},
	{ PLX_REG(DMACSR1)	},
	{ PLX_REG(DMAARB)	},
	{ PLX_REG(DMATHR)	},

	{ PLX_REG(OPLFIS)	},
	{ PLX_REG(OPLFIM)	},
	{ PLX_REG(IQP)		},
	{ PLX_REG(OQP)		},
	{ PLX_REG(MQCR)		},
	{ PLX_REG(QBAR)		},
	{ PLX_REG(IFHPR)	},
	{ PLX_REG(IFTPR)	},
	{ PLX_REG(IPHPR)	},
	{ PLX_REG(IPTPR)	},
	{ PLX_REG(OFHPR)	},
	{ PLX_REG(OFTPR)	},
	{ PLX_REG(OPHPR)	},
	{ PLX_REG(OPTPR)	},
	{ PLX_REG(QSR)		},
	{ NULL, 0			}
};

#define	GSC_REG(r)	#r, HPDI32_GSC_##r

static const reg_entry_t	_gsc_regs[]	=
{
	{ GSC_REG(FRR)		},
	{ GSC_REG(BCR)		},
	{ GSC_REG(BSR)		},
	{ GSC_REG(TAR)		},
	{ GSC_REG(RAR)		},
	{ GSC_REG(TXODPR)	},
	{ GSC_REG(FDR)		},
	{ GSC_REG(TXMLSR)	},
	{ GSC_REG(TXMLCR)	},
	{ GSC_REG(TXPDLR)	},
	{ GSC_REG(RXIDPR)	},
	{ GSC_REG(RXMLSR)	},
	{ GSC_REG(ICR)		},
	{ GSC_REG(ISR)		},
	{ GSC_REG(IELR)		},
	{ GSC_REG(IHLR)		},
	{ NULL, 0			}
};



/*****************************************************************************
*
*	Function:	_dump_regs
*
*	Purpose:
*
*		Dump the contents of the specified registers.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		title	The name for the registers being dumped.
*
*		list	The list of registers to dump.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _dump_regs(int fd, const char* title, const reg_entry_t* list)
{
	int		errs	= 0;
	int		status;
	__u32	value;

	label(title);
	printf("\n");
	label_level_inc();

	for (; list->name; list++)
	{
		label(list->name);

		status	= hpdi32_reg_read(fd, list->reg, &value, 1);
		errs	+= status ? 1 : 0;
		printf("0x%08lX\n", (long) value);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	register_dump
*
*	Purpose:
*
*		Dump the contents of the specified registers.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		pci		Dump the PCI register?
*
*		plx		Dump the PLX registers?
*
*		gsc		Dump the GSC registers?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int register_dump(int fd, int pci, int plx, int gsc)
{
	int		errs	= 0;

	label("Register Dump");
	printf("\n");
	label_level_inc();

	if (pci)
		errs	+= _dump_regs(fd, "PCI Registers", _pci_regs);

	if (plx)
		errs	+= _dump_regs(fd, "PLX Registers", _plx_regs);

	if (gsc)
		errs	+= _dump_regs(fd, "GSC Registers", _gsc_regs);

	label_level_dec();
	return(errs);
}



#if 0
/*****************************************************************************
*
*	Function:	register_read_test
*
*	Purpose:
*
*		Verify that all readable registers can be read.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

int register_read_test(int fd)
{
	int		i;
	__u32	reg;
	int		status	= 0;

	label("Register Read");

	for (i = 0; (status == 0) && (i < ARRAY_ELEMENTS(_reg_list)); i++)
		status	= hpdi32_reg_read(fd, _reg_list[i].reg, &reg, 0);

	printf("%s (%d)\n", (status == -1) ? "FAIL" : "PASS", i);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}



/*****************************************************************************
*
*	Function:	register_write_test
*
*	Purpose:
*
*		Verify that all writeable registers can be written.
*
*	Arguments:
*
*		fd	The handle of the board to access.
*
*	Returned:
*
*		0	All went well.
*		-1	There was a problem, see errno.
*
*****************************************************************************/

int register_write_test(int fd)
{
	int		i;
	__u32	reg;
	int		ro		= 0;
	int		rw		= 0;
	int		status	= 0;

	label("Register Write");

	for (i = 0; (status == 0) && (i < ARRAY_ELEMENTS(_reg_list)); i++)
	{
		hpdi32_reg_read(fd, _reg_list[i].reg, &reg, 0);
		status	= hpdi32_reg_write(fd, _reg_list[i].reg, reg, 0);

		if (((_reg_list[i].writeable) && (status)) ||
			((!_reg_list[i].writeable) && (!status)))
		{
			/* If its RW then there should be no error. */
			/* It its RO then there should be an error. */
			status	= 1;
			break;
		}
		else
		{
			status	= 0;

			if (_reg_list[i].writeable)
				rw++;
			else
				ro++;
		}
	}

	printf(	"%s (%d: rw %d, ro %d)\n",
			(status == -1) ? "FAIL" : "PASS",
			i,
			rw,
			ro);
	hpdi32_reg_write(fd, HPDI32_GSC_BCR, HPDI32_GSC_BCR_BOARD_RESET, 0);
	return(status);
}


#endif

