/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* $Revision: 2 $ */
/* $Date: 8/16/06 11:57a $ */
/* $Log: /NEW SOURCESAFE DATABASE/DIGITAL/Drivers/LINUX/HPDI32/HPDI32_Linux_Ver.1.16.0-GSC-DN/diphase2/sbtest/tx.c $
 *
 * 2     8/16/06 11:57a Dnewbold
 * Did some code linting.
 *
 * 1     2/22/06 10:08a Dnewbold
*/

#include <stdio.h>

#include "hpdi32_diphase2.h"
#include "main.h"
#include "HPDI32DocSrcLib.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	_tx_config_enable_get_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Enable Get service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_enable_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	big;
	int	enable;
	int	errs	= 0;
	int	i;
	__u32	length;
	int	raw;
	int	start;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_ENABLE);
		errs	|= hpdi32_dp2_tx_config_get(fd,
											&enable,
											&start,
											&big,
											&raw,
											&length);

		// Verify that Enable changes as expected.
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that Tx Start is unaffected.
		errs	|= start ? 1 : 0;

		// Verify that Big Endian is unaffected.
		errs	|= big ? 1 : 0;

		// Verify that Raw Output is unaffected.
		errs	|= raw ? 1 : 0;

		// Verify that Message Length is unaffected.
		errs	|= length ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_enable_set_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Enable Set service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_enable_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		big;
	int		enable;
	int		errs	= 0;
	int		i;
	int		raw;
	int		start;
	__u32	txmlcr;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_tx_config_set(fd, list[i].set, -1, -1, -1, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXMLCR, &txmlcr);

		// Verify that Enable changes as expected.
		enable	= (bcr & HPDI32_GSC_BCR_TX_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].get) ? 0 : 1;

		// Verify that Tx Start is unaffected.
		start	= (bcr & HPDI32_GSC_BCR_TX_START) ? 1 : 0;
		errs	|= start ? 1 : 0;

		// Verify that Message Length is unaffected.
		big		= (bcr & HPDI32_GSC_BCR_TX_BIG_ENDIAN) ? 1 : 0;
		errs	|= big ? 1 : 0;

		// Verify that Raw Output is unaffected.
		raw		= (bcr & HPDI32_GSC_BCR_TX_RAW_OUTPUT) ? 1 : 0;
		errs	|= raw ? 1 : 0;

		// Verify that Message Length is unaffected.
		errs	|= txmlcr ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_enable_reset_tests
*
*	Purpose:
*
*		Perform reset tests on the library's Tx Configuration Enable feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_enable_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		enable;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_ENABLE);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_TX_ENABLE) ? 1 : 0;
		errs	|= (enable == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		enable	= (bcr & HPDI32_GSC_BCR_TX_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_enable_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Enable feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_enable_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Enable Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _tx_config_enable_get_tests(fd, verbose);
	errs	|= _tx_config_enable_set_tests(fd, verbose);
	errs	|= _tx_config_enable_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_tx_start_get_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Tx Start Get service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_tx_start_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	big;
	int	enable;
	int	errs	= 0;
	int	i;
	__u32	length;
	int	raw;
	int	start;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_START);
		errs	|= hpdi32_dp2_tx_config_get(fd,
											&enable,
											&start,
											&big,
											&raw,
											&length);

		// Verify that Tx Start changes as expected.
		errs	|= (start == list[i].set) ? 0 : 1;

		// Verify that Enable is unaffected.
		errs	|= enable ? 1 : 0;

		// Verify that Big Endian is unaffected.
		errs	|= big ? 1 : 0;

		// Verify that Raw Output is unaffected.
		errs	|= raw ? 1 : 0;

		// Verify that Message Length is unaffected.
		errs	|= length ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_tx_start_set_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Tx Start Set service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_tx_start_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		big;
	int		enable;
	int		errs	= 0;
	int		i;
	int		raw;
	int		start;
	__u32	txmlcr;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_tx_config_set(fd, -1, list[i].set, -1, -1, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXMLCR, &txmlcr);

		// Verify that Tx Start changes as expected.
		start	= (bcr & HPDI32_GSC_BCR_TX_START) ? 1 : 0;
		errs	|= (start == list[i].get) ? 0 : 1;

		// Verify that Enable is unaffected.
		enable	= (bcr & HPDI32_GSC_BCR_TX_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that Big Endian is unaffected.
		big		= (bcr & HPDI32_GSC_BCR_TX_BIG_ENDIAN) ? 1 : 0;
		errs	|= big ? 1 : 0;

		// Verify that Raw Output is unaffected.
		raw		= (bcr & HPDI32_GSC_BCR_TX_RAW_OUTPUT) ? 1 : 0;
		errs	|= raw ? 1 : 0;

		// Verify that Message Length is unaffected.
		errs	|= txmlcr ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_tx_start_reset_tests
*
*	Purpose:
*
*		Perform reset tests on the library's Tx Configuration Tx Start
*		feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_tx_start_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		errs	= 0;
	int		i;
	int		start;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_START);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		start	= (bcr & HPDI32_GSC_BCR_TX_START) ? 1 : 0;
		errs	|= (start == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		start	= (bcr & HPDI32_GSC_BCR_TX_START) ? 1 : 0;
		errs	|= start ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_tx_start_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Tx Start feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_tx_start_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Tx Start Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _tx_config_tx_start_get_tests(fd, verbose);
	errs	|= _tx_config_tx_start_set_tests(fd, verbose);
	errs	|= _tx_config_tx_start_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_rx_config_big_endian_get_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Big Endian Get service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_big_endian_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	big;
	int	enable;
	int	errs	= 0;
	int	i;
	__u32	length;
	int	raw;
	int	start;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_BIG_ENDIAN);
		errs	|= hpdi32_dp2_tx_config_get(fd,
											&enable,
											&start,
											&big,
											&raw,
											&length);

		// Verify that Big Endian changes as expected.
		errs	|= (big == list[i].set) ? 0 : 1;

		// Verify that Enable is unaffected.
		errs	|= enable ? 1 : 0;

		// Verify that Tx Start is unaffected.
		errs	|= start ? 1 : 0;

		// Verify that Raw Output is unaffected.
		errs	|= raw ? 1 : 0;

		// Verify that Message Length is unaffected.
		errs	|= length ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_big_endian_set_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Big Endian Set service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_big_endian_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		big;
	int		enable;
	int		errs	= 0;
	int		i;
	int		raw;
	int		start;
	__u32	txmlcr;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_tx_config_set(fd, -1, -1, list[i].set, -1, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXMLCR, &txmlcr);

		// Verify that Big Endian changes as expected.
		big		= (bcr & HPDI32_GSC_BCR_TX_BIG_ENDIAN) ? 1 : 0;
		errs	|= (big == list[i].get) ? 0 : 1;

		// Verify that Enable is unaffected.
		enable	= (bcr & HPDI32_GSC_BCR_TX_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that Tx Start is unaffected.
		start	= (bcr & HPDI32_GSC_BCR_TX_START) ? 1 : 0;
		errs	|= start ? 1 : 0;

		// Verify that Tx Start is unaffected.
		raw		= (bcr & HPDI32_GSC_BCR_TX_RAW_OUTPUT) ? 1 : 0;
		errs	|= raw ? 1 : 0;

		// Verify that Message Length is unaffected.
		errs	|= txmlcr ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_big_endian_reset_tests
*
*	Purpose:
*
*		Perform reset tests on the library's Tx Configuration Big Endian
*		feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_big_endian_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		big;
	int		errs	= 0;
	int		i;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_BIG_ENDIAN);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		big		= (bcr & HPDI32_GSC_BCR_TX_BIG_ENDIAN) ? 1 : 0;
		errs	|= (big == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		big		= (bcr & HPDI32_GSC_BCR_TX_BIG_ENDIAN) ? 1 : 0;
		errs	|= big ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_big_endian_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Big Endian feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_big_endian_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Big Endian Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _tx_config_big_endian_get_tests(fd, verbose);
	errs	|= _tx_config_big_endian_set_tests(fd, verbose);
	errs	|= _tx_config_big_endian_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_rx_config_raw_output_get_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Raw Output Get service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_raw_output_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	int	big;
	int	enable;
	int	errs	= 0;
	int	i;
	__u32	length;
	int	raw;
	int	start;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_RAW_OUTPUT);
		errs	|= hpdi32_dp2_tx_config_get(fd,
											&enable,
											&start,
											&big,
											&raw,
											&length);

		// Verify that Raw Output changes as expected.
		errs	|= (raw == list[i].set) ? 0 : 1;

		// Verify that Enable is unaffected.
		errs	|= enable ? 1 : 0;

		// Verify that Tx Start is unaffected.
		errs	|= start ? 1 : 0;

		// Verify that Big Endian is unaffected.
		errs	|= big ? 1 : 0;

		// Verify that Message Length is unaffected.
		errs	|= length ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_raw_output_set_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Raw Output Set service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_raw_output_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0	},
		{ 1,	1	},
		{ -1,	1	},
		{ 0,	0	},
		{ -1,	0	},
		{ -1,	-1	}
	};

	__u32	bcr;
	int		big;
	int		enable;
	int		errs	= 0;
	int		i;
	int		raw;
	int		start;
	__u32	txmlcr;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_tx_config_set(fd, -1, -1, -1, list[i].set, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXMLCR, &txmlcr);

		// Verify that Raw Output changes as expected.
		raw		= (bcr & HPDI32_GSC_BCR_TX_RAW_OUTPUT) ? 1 : 0;
		errs	|= (raw == list[i].get) ? 0 : 1;

		// Verify that Enable is unaffected.
		enable	= (bcr & HPDI32_GSC_BCR_TX_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that Tx Start is unaffected.
		start	= (bcr & HPDI32_GSC_BCR_TX_START) ? 1 : 0;
		errs	|= start ? 1 : 0;

		// Verify that Big Endian is unaffected.
		big		= (bcr & HPDI32_GSC_BCR_TX_BIG_ENDIAN) ? 1 : 0;
		errs	|= big ? 1 : 0;

		// Verify that Message Length is unaffected.
		errs	|= txmlcr ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_raw_output_reset_tests
*
*	Purpose:
*
*		Perform reset tests on the library's Tx Configuration Raw Output
*		feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_raw_output_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 0		},
		{ 1		},
		{ 0		},
		{ -1	}
	};

	__u32	bcr;
	int		errs	= 0;
	int		i;
	int		raw;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										list[i].set ? ~0L : 0,
										HPDI32_GSC_BCR_TX_RAW_OUTPUT);

		// Verify that the setting is unchanged by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		raw		= (bcr & HPDI32_GSC_BCR_TX_RAW_OUTPUT) ? 1 : 0;
		errs	|= (raw == list[i].set) ? 0 : 1;

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		raw		= (bcr & HPDI32_GSC_BCR_TX_RAW_OUTPUT) ? 1 : 0;
		errs	|= raw ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_raw_output_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Raw Output feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_raw_output_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Raw Output Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _tx_config_raw_output_get_tests(fd, verbose);
	errs	|= _tx_config_raw_output_set_tests(fd, verbose);
	errs	|= _tx_config_raw_output_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_length_get_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Message Length Get
*		service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_length_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 100	},
		{ 1000	},
		{ 2560	},
		{ 9999	},
		{ -1	}
	};

	int	big;
	int	enable;
	int	errs	= 0;
	int	i;
	__u32	length;
	int	raw;
	int	start;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_TXMLCR, list[i].set);
		errs	|= hpdi32_dp2_tx_config_get(fd,
											&enable,
											&start,
											&big,
											&raw,
											&length);

		// Verify that Message Length changes as expected.
		errs	|= (length == list[i].set) ? 0 : 1;

		// Verify that Enable is unaffected.
		errs	|= enable ? 1 : 0;

		// Verify that Tx Start is unaffected.
		errs	|= start ? 1 : 0;

		// Verify that Big Endian is unaffected.
		errs	|= big ? 1 : 0;

		// Verify that Raw Output is unaffected.
		errs	|= raw ? 1 : 0;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_length_set_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Message Length Set
*		service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_length_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	0		},
		{ 100,	100		},
		{ -1,	100		},
		{ 2000,	2000	},
		{ -1,	2000	},
		{ -1,	-1		}
	};

	__u32	bcr;
	int		big;
	int		enable;
	int		errs	= 0;
	int		i;
	int		raw;
	int		start;
	__u32	txmlcr;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_tx_config_set(fd, -1, -1, -1, -1, list[i].set);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BCR, &bcr);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TXMLCR, &txmlcr);

		// Verify that Message Length changes as expected.
		errs	|= (txmlcr == list[i].get) ? 0 : 1;

		// Verify that Enable is unaffected.
		enable	= (bcr & HPDI32_GSC_BCR_TX_ENABLE) ? 1 : 0;
		errs	|= enable ? 1 : 0;

		// Verify that Tx Start is unaffected.
		start	= (bcr & HPDI32_GSC_BCR_TX_START) ? 1 : 0;
		errs	|= start ? 1 : 0;

		// Verify that Big Endian is unaffected.
		big		= (bcr & HPDI32_GSC_BCR_TX_BIG_ENDIAN) ? 1 : 0;
		errs	|= big ? 1 : 0;

		// Verify that Raw Output is unaffected.
		raw		= (bcr & HPDI32_GSC_BCR_TX_RAW_OUTPUT) ? 1 : 0;
		errs	|= raw ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_length_reset_tests
*
*	Purpose:
*
*		Perform reset tests on the library's Tx Configuration Message Length
*		feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_length_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 10	},
		{ 1000	},
		{ 10000	},
		{ -1	}
	};

	int		errs	= 0;
	int		i;
	__u32	txmlcr;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting.
		errs	|= hpdi32_dp2_reg_write(fd,	HPDI32_GSC_TXMLCR, list[i].set);

		// Verify that the setting is reset by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd,	HPDI32_GSC_TXMLCR, &txmlcr);
		errs	|= txmlcr ? 1 : 0;

		// Apply a setting.
		errs	|= hpdi32_dp2_reg_write(fd,	HPDI32_GSC_TXMLCR, list[i].set);

		// Verify that the setting is reset by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd,	HPDI32_GSC_TXMLCR, &txmlcr);
		errs	|= txmlcr ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_length_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration Message Length feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_length_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Message Length Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _tx_config_length_get_tests(fd, verbose);
	errs	|= _tx_config_length_set_tests(fd, verbose);
	errs	|= _tx_config_length_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_config_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx Configuration services.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_config_tests(int fd)
{
	int	errs	= 0;

	label("Tx Configuration");
	printf("\n");

	label_level_inc();

	errs	|= _tx_config_enable_tests(fd);
	errs	|= _tx_config_tx_start_tests(fd);
	errs	|= _tx_config_big_endian_tests(fd);
	errs	|= _tx_config_raw_output_tests(fd);
	errs	|= _tx_config_length_tests(fd);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_ae_get_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx FIFO Empty Full Get feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_ae_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 100	},
		{ 256	},
		{ 128	},
		{ 0		},
		{ -1	}
	};

	int	ae;
	int	af;
	int	errs	= 0;
	int	i;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TAR,
										(long) list[i].set,
										0x0000FFFF);
		errs	|= hpdi32_dp2_tx_fifo_almost_get(fd, &af, &ae);

		// Verify that Almost Empty changes as expected.
		errs	|= (ae == list[i].set) ? 0 : 1;

		// Verify that Almost Full is unaffected.
		errs	|= (af == 256) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_ae_set_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx FIFO Almost Empty Set feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_ae_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	8	},
		{ 0,	0	},
		{ -1,	0	},
		{ 100,	100	},
		{ -1,	100	},
		{ 128,	128	},
		{ -1,	128	},
		{ -1,	-1	}
	};

	int		ae;
	int		af;
	__u32	bsr;
	int		errs	= 0;
	int		i;
	__u32	tar;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_tx_fifo_almost_set(fd, -1, list[i].set);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TAR, &tar);

		// Verify that Almost Empty changes as expected.
		ae		= HPDI32_GSC_TAR_DECODE_EMPTY(tar);
		errs	|= (ae == list[i].get) ? 0 : 1;

		// Verify that Almost Full is unchanged.
		af		= HPDI32_GSC_TAR_DECODE_FULL(tar);
		errs	|= (af == 256) ? 0 : 1;
	}

	// Now verify that the Set service actually resets the FIFO.
	errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);
	errs	|= (bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) ? 1 : 0;
	errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_FDR, 1);
	errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);
	errs	|= (bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) ? 0 : 1;
	errs	|= hpdi32_dp2_tx_fifo_almost_set(fd, 16, 16);
	errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);
	errs	|= (bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) ? 1 : 0;

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_ae_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's Tx FIFO Almost Empty feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_ae_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 100	},
		{ 1001	},
		{ 0		},
		{ -1	}
	};

	int		ae;
	int		errs	= 0;
	int		i;
	__u32	tar;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TAR,
										(long) list[i].set,
										0x0000FFFF);

		// Verify that the setting is reset by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TAR, &tar);
		ae		= HPDI32_GSC_TAR_DECODE_EMPTY(tar);
		errs	|= (ae == 8) ? 0 : 1;

		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TAR,
										(long) list[i].set,
										0x0000FFFF);

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TAR, &tar);
		ae		= HPDI32_GSC_TAR_DECODE_EMPTY(tar);
		errs	|= (ae == 8) ? 0 : 1;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_ae_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx FIFO Almost Empty feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_ae_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Almost Empty Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _tx_fifo_ae_get_tests(fd, verbose);
	errs	|= _tx_fifo_ae_set_tests(fd, verbose);
	errs	|= _tx_fifo_ae_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_af_get_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx FIFO Almost Full Get feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_af_get_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 0		},
		{ 100	},
		{ 256	},
		{ 128	},
		{ 0		},
		{ -1	}
	};

	int	ae;
	int	af;
	int	errs	= 0;
	int	i;

	if (verbose)
		label("Get Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TAR,
										(long) list[i].set << 16,
										0xFFFF0000);
		errs	|= hpdi32_dp2_tx_fifo_almost_get(fd, &af, &ae);

		// Verify that Almost Full changes as expected.
		errs	|= (af == list[i].set) ? 0 : 1;

		// Verify that Almost Empty is unaffected.
		errs	|= (ae == 8) ? 0 : 1;
	}

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_af_set_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx FIFO Almost Full Set feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_af_set_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
		int	get;
	} list[]	=
	{
		// set	get
		{ -1,	256	},
		{ 0,	0	},
		{ -1,	0	},
		{ 100,	100	},
		{ -1,	100	},
		{ 128,	128	},
		{ -1,	128	},
		{ -1,	-1	}
	};

	int		ae;
	int		af;
	__u32	bsr;
	int		errs	= 0;
	int		i;
	__u32	tar;

	if (verbose)
		label("Set Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (i = 0; list[i].get >= 0; i++)
	{
		// Apply a setting change and check the results.
		errs	|= hpdi32_dp2_tx_fifo_almost_set(fd, list[i].set, -1);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TAR, &tar);

		// Verify that Almost Full changes as expected.
		af		= HPDI32_GSC_TAR_DECODE_FULL(tar);
		errs	|= (af == list[i].get) ? 0 : 1;

		// Verify that Almost Empty is unchanged.
		ae		= HPDI32_GSC_TAR_DECODE_EMPTY(tar);
		errs	|= (ae == 8) ? 0 : 1;
	}

	// Now verify that the Set service actually resets the FIFO.
	errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);
	errs	|= (bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) ? 1 : 0;
	errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_FDR, 1);
	errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);
	errs	|= (bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) ? 0 : 1;
	errs	|= hpdi32_dp2_tx_fifo_almost_set(fd, 16, 16);
	errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);
	errs	|= (bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) ? 1 : 0;

	errs	|= hpdi32_dp2_board_reset(fd);

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_af_reset_tests
*
*	Purpose:
*
*		Perform reset tests of the library's Tx FIFO Almost Full feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_af_reset_tests(int fd, int verbose)
{
	static const struct
	{
		int	set;
	} list[]	=
	{
		// set
		{ 1		},
		{ 100	},
		{ 1001	},
		{ 0		},
		{ -1	}
	};

	int		af;
	int		errs	= 0;
	int		i;
	__u32	tar;

	if (verbose)
		label("Reset");

	for (i = 0; list[i].set >= 0; i++)
	{
		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TAR,
										(long) list[i].set << 16,
										0xFFFF0000);

		// Verify that the setting is reset by the Board Reset bit.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_BCR,
										~0L,
										HPDI32_GSC_BCR_BOARD_RESET);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TAR, &tar);
		af		= HPDI32_GSC_TAR_DECODE_FULL(tar);
		errs	|= (af == 256) ? 0 : 1;

		// Apply a bit setting.
		errs	|= hpdi32_dp2_reg_mod(	fd,
										HPDI32_GSC_TAR,
										(long) list[i].set << 16,
										0xFFFF0000);

		// Verify that the setting is cleared by the Board Reset service.
		errs	|= hpdi32_dp2_board_reset(fd);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_TAR, &tar);
		af		= HPDI32_GSC_TAR_DECODE_FULL(tar);
		errs	|= (af == 256) ? 0 : 1;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_af_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx FIFO Almost Full feature.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_af_tests(int fd)
{
	int	errs	= 0;
	int	verbose	= 0;

	label("Almost Full Setting");

	if (verbose)
		printf("\n");

	label_level_inc();

	errs	|= _tx_fifo_af_get_tests(fd, verbose);
	errs	|= _tx_fifo_af_set_tests(fd, verbose);
	errs	|= _tx_fifo_af_reset_tests(fd, verbose);

	label_level_dec();

	if (verbose == 0)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_reset_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx FIFO Reset service.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_reset_tests(int fd)
{
	__u32	bsr;
	int		errs	= 0;

	label("Reset Service");

	errs	|= hpdi32_dp2_board_reset(fd);

	// Put data into the Tx FIFO.
	errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_FDR, 1);
	errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);
	errs	|= (bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) ? 0 : 1;

	// Reset the Tx FIFO and check the results.
	errs	|= hpdi32_dp2_tx_fifo_reset(fd);

	// Verify that the Tx FIFO is empty.
	errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);
	errs	|= (bsr & HPDI32_GSC_BSR_TX_NOT_EMPTY) ? 1 : 0;

	errs	|= hpdi32_dp2_board_reset(fd);
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_size_tests
*
*	Purpose:
*
*		Perform tests to compute the Tx FIFO size.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_size_tests(int fd)
{
	__u32	bsr;
	int		errs	= 0;
	int		size;

	label("FIFO Size");

	errs	|= hpdi32_dp2_board_reset(fd);

	for (size = 1; errs == 0; size *= 2)
	{
		errs	|= hpdi32_dp2_tx_fifo_almost_set(fd, size - 1, 1);
		errs	|= hpdi32_dp2_reg_write(fd, HPDI32_GSC_FDR, 0);
		errs	|= hpdi32_dp2_reg_read(fd, HPDI32_GSC_BSR, &bsr);

		if ((bsr & HPDI32_GSC_BSR_TX_NOT_AF) == 0)
			break;
	}

	if (errs)
		printf("FAIL <---\n");
	else
		printf("%d Samples Deep\n", size);

	return(errs);
}



/*****************************************************************************
*
*	Function:	_tx_fifo_tests
*
*	Purpose:
*
*		Perform tests on the library's Tx FIFO features.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _tx_fifo_tests(int fd)
{
	int	errs	= 0;

	label("Tx FIFO");
	printf("\n");

	label_level_inc();

	errs	|= _tx_fifo_reset_tests(fd);
	errs	|= _tx_fifo_af_tests(fd);
	errs	|= _tx_fifo_ae_tests(fd);
	errs	|= _tx_fifo_size_tests(fd);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	tx_tests
*
*	Purpose:
*
*		Perform tests on the transmitter.
*
*	Arguments:
*
*		fd		The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int tx_tests(int fd)
{
	int		errs	= 0;

	label("Transmitter Tests");
	printf("\n");
	label_level_inc();

	errs	|= _tx_fifo_tests(fd);
	errs	|= _tx_config_tests(fd);

	label_level_dec();
	return(errs);
}



